/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package org.glite.ce.creamapi.cmdmanagement;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.glite.ce.creamapi.cmdmanagement.queue.CommandQueueInterface;

public abstract class AbstractCommandExecutor implements CommandExecutor {
    private static final Logger logger = Logger.getLogger(AbstractCommandExecutor.class.getName());

    private CommandQueueInterface commandQueue;
    private List<CommandWorker> threadPool = null;
    private String[] commands;
    private String category;
    private String name;
    private Hashtable<String, Object> parameter;
    private CommandManagerInterface commandManager;
    
    protected ThreadGroup poolGroup = new ThreadGroup("Manager Worker Threads");

    protected AbstractCommandExecutor(String name, String category) {
        this(name, category, null);
    }

    protected AbstractCommandExecutor(String name, String category, String[] commands) {
        this.name = name;
        this.category = category;
        this.commands = commands;
        parameter = new Hashtable<String, Object>(0);
    }

    public void addParameter(String key, Object value) {
        if (key != null && value != null) {
            parameter.put(key, value);
        }
    }

    public boolean checkCommandSupport(String name) {
        if (name == null) {
            return false;
        }

        if (commands != null) {
            for (int i = 0; i < commands.length; i++) {
                if (commands[i].equals(name)) {
                    return true;
                }
            }
        }
        return false;
    }

    public boolean containsParameterKey(String key) {
        if (key == null) {
            return false;
        }
        return parameter.containsKey(key);
    }

    public void destroy() {
        logger.info("destroy invoked!");

        if (commandQueue != null) {
            commandQueue.close();

            do {
                for (int i = 0; i < threadPool.size(); i++) {
                    if(!threadPool.get(i).isProcessing() && !threadPool.get(i).isInterrupted()) {
                        threadPool.get(i).destroy();
                        threadPool.remove(i);
                    }
                }

                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                }
            } while(threadPool.size() > 0);

            commandQueue.destroy();
        }

        commandQueue = null;
        commandManager = null;
        logger.info("destroyed!");
    }

    public abstract void execute(Command cmd) throws CommandExecutorException, CommandException, IllegalArgumentException;

    public abstract void execute(Command[] cmd) throws CommandExecutorException, CommandException, IllegalArgumentException;

    public String getCategory() {
        return category;
    }

    public CommandManagerInterface getCommandManager() {
        return commandManager;
    }
    
    public CommandQueueInterface getCommandQueue() {
        return commandQueue;
    }

    public String[] getCommands() {
        return commands;
    }

    public String getName() {
        return name;
    }

    public Set getParameterKeySet() {
        return parameter.keySet();
    }

    public Object getParameterValue(String key) {
        if (key != null) {
            return parameter.get(key);
        }

        return null;
    }

    public String getParameterValueAsString(String key) {
        if (key != null) {
            return (String) parameter.get(key);
        }

        return null;
    }

    public void setCategory(String category) {
        this.category = category;
    }

    public void setCommandManager(CommandManagerInterface cmdManager) {
        this.commandManager = cmdManager;
    }

    public void setCommandQueue(CommandQueueInterface queue) {
        commandQueue = queue;
        // setCommandWorkerPoolSize(3);
    }

    public void setCommands(String[] cmd) {
        commands = cmd;
    }

    public void setCommandWorkerPoolSize(int size) {
        // Initialize the thread pool
        threadPool = new ArrayList<CommandWorker>(size);
        CommandWorker cw = null;
        for (int i = 0; i < size; i++) {
            cw = new CommandWorker(this, i);
            cw.start();
            
            threadPool.add(cw);
        }
    }

    public void setName(String name) {
        this.name = name;
    }
}
