/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Luigi Zangrando, <luigi.zangrando@pd.infn.it>
 *
 */

package org.glite.ce.creamapi.delegationmanagement;

import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.Security;
import java.security.cert.X509Certificate;
import java.text.FieldPosition;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.TimeZone;

import org.apache.log4j.Logger;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.glite.security.delegation.GrDPX509Util;
import org.glite.security.util.DNHandler;
import org.glite.voms.VOMSAttribute;
import org.glite.voms.VOMSValidator;

public class Delegation implements Cloneable {
    private static final long serialVersionUID = 1L;

    private static Logger logger = Logger.getLogger(Delegation.class.getName());

    static {
        if (Security.getProvider("BC") == null) {
            Security.addProvider(new BouncyCastleProvider());
        }
    }

    public static Delegation getInstance(String id) throws IllegalArgumentException {
        return new Delegation(id);
    }

    public static Delegation getInstance(String id, String proxy) throws IllegalArgumentException {
        if (proxy == null) {
            throw (new IllegalArgumentException("Missing proxy certificate"));
        }

        Delegation delegation = new Delegation(id, proxy);

        try {
            X509Certificate[] userCertChain = GrDPX509Util.loadCertificateChain(proxy.getBytes("US-ASCII"));

            if (userCertChain == null || userCertChain.length == 0) {

            }

            VOMSValidator mainValidator = new VOMSValidator(userCertChain);
            mainValidator.validate();

            X509Certificate certificate = null;
            int userCertIndex = 0;
            int proxyCertIndex = 0;
            int userCertDNLength = Integer.MAX_VALUE;
            int proxyCertDNLength = Integer.MIN_VALUE;
            int DNLength = 0;

            for (int index = 0; index < userCertChain.length; index++) {
                certificate = userCertChain[index];

                // if !(CA or subCA)
                if (!(certificate.getBasicConstraints() > -1)) {
                    DNLength = (DNHandler.getDN(certificate.getSubjectDN().toString()).getX500().length());
                    if (DNLength < userCertDNLength) {
                        userCertDNLength = DNLength;
                        userCertIndex = index;
                    }

                    if (proxyCertDNLength < DNLength) {
                        proxyCertDNLength = DNLength;
                        proxyCertIndex = index;
                    }

                }
            }

            String userDN = DNHandler.getDN(userCertChain[userCertIndex].getSubjectDN().toString()).getX500();
            String issuerDN = DNHandler.getDN(userCertChain[userCertIndex].getIssuerDN().toString()).getX500();

            delegation.setDN(userDN);
            delegation.setStartTime(userCertChain[proxyCertIndex].getNotBefore());
            delegation.setExpirationTime(userCertChain[proxyCertIndex].getNotAfter());
            logger.debug("Found subject DN " + userDN + " and issuer DN " + issuerDN);

            SimpleDateFormat dateFormat = new SimpleDateFormat();
            dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
            FieldPosition fp = new FieldPosition(SimpleDateFormat.YEAR_FIELD);

            StringBuffer buff = new StringBuffer("Valid From      : ");
            dateFormat.format(userCertChain[proxyCertIndex].getNotBefore(), buff, fp);
            buff.append(" (GMT)\nValid To       : ");
            dateFormat.format(userCertChain[proxyCertIndex].getNotAfter(), buff, fp);
            buff.append(" (GMT)");
            buff.append("\nHolder Subject : ").append(userDN);
            buff.append("\nHolder CA      : ").append(issuerDN);
            buff.append("\n");

            HashMap<String, List<String>> proxyVOAttrs = new HashMap<String, List<String>>(0);
            List<String> vomsAttrituteList = new ArrayList<String>(0);

            for (VOMSAttribute vomsAttr : (List<VOMSAttribute>) mainValidator.getVOMSAttributes()) {
                buff.append("\nVO              : ").append(vomsAttr.getVO());
                buff.append("\nAC Issuer       : ").append(vomsAttr.getIssuer());
                buff.append("\nAttribute       : ");

                for (String attr : (List<String>) vomsAttr.getFullyQualifiedAttributes()) {
                    buff.append(attr).append(" ");
                    vomsAttrituteList.add(attr);
                }

                buff.append("\n");

                proxyVOAttrs.put(vomsAttr.getVO(), vomsAttr.getFullyQualifiedAttributes());
            }

            if (!proxyVOAttrs.values().isEmpty()) {
                List<String> fqanList = proxyVOAttrs.values().iterator().next();
                if (fqanList != null && fqanList.size() > 0) {
                    delegation.setFQAN(fqanList.get(0).toString());
                }
            }

            if (!proxyVOAttrs.keySet().isEmpty()) {
                delegation.setVO(proxyVOAttrs.keySet().iterator().next());
            }

            delegation.setInfo(buff.toString());
            if (!vomsAttrituteList.isEmpty()) {
                delegation.setVOMSAttributes(vomsAttrituteList);
            }
            // delegation.setVomsAttributes(mainValidator.getAllFullyQualifiedAttributes());

            // mainValidator.cleanup();
        } catch (IOException e) {
            throw new RuntimeException(e.getMessage());
        } catch (GeneralSecurityException e) {
            throw new RuntimeException(e.getMessage());
        }

        return delegation;
    }

    private String id = null;
    private String info = "N/A";
    private String proxy = null;
    private String userVO = null;
    private String userFQAN = null;
    private String userDN = null;
    private String localUser = null;
    private String localUserGroup = null;
    private List<String> vomsAttributes = null;
    private Date startTime = null;
    private Date expirationTime = null;
    private Date lastUpdateTime = null;

    protected Delegation(String id) {
        this.id = id;
    }

    protected Delegation(String id, String proxy) throws IllegalArgumentException {
        this.id = id;
        this.proxy = proxy;
    }

    public Object clone() {
        Delegation delegation = new Delegation(id, proxy);
        delegation.setInfo(info);
        delegation.setDN(userDN);
        delegation.setFQAN(userFQAN);
        delegation.setVO(userVO);
        delegation.setLocalUser(localUser);
        delegation.setLocalUserGroup(localUserGroup);
        delegation.setVOMSAttributes(vomsAttributes);
        delegation.setExpirationTime(expirationTime);
        delegation.setLastUpdateTime(lastUpdateTime);
        delegation.setStartTime(startTime);

        return delegation;
    }

    public String getDN() {
        return userDN;
    }

    public Date getExpirationTime() {
        return expirationTime;
    }

    public String getFQAN() {
        return userFQAN;
    }

    public String getId() {
        return id;
    }

    public String getInfo() {
        return info;
    }

    public Date getLastUpdateTime() {
        return lastUpdateTime;
    }

    public String getLocalUser() {
        return localUser;
    }

    public String getLocalUserGroup() {
        return localUserGroup;
    }

    public String getProxy() {
        return proxy;
    }

    // private X509Certificate[] getProxyChain() throws IOException,
    // GeneralSecurityException {
    // if (proxy == null) {
    // throw new GeneralSecurityException("Proxy is empty");
    // }
    // return GrDPX509Util.loadCertificateChain(proxy.getBytes("US-ASCII"));
    // }

    public Date getStartTime() {
        return startTime;
    }

    public String getUserId() {
        if (userFQAN != null) {
            return normalize(userDN + userFQAN);
        }

        return normalize(userDN);
    }

    public String getVO() {
        return userVO;
    }

    public List<String> getVOMSAttributes() {
        return vomsAttributes;
    }

    public boolean isValid() {
        return System.currentTimeMillis() < expirationTime.getTime();
    }

    private String normalize(String s) {
        if (s != null) {
            return s.replaceAll("\\W", "_");
        }
        return null;
    }

    public void setDN(String userDN) {
        this.userDN = userDN;
    }

    public void setExpirationTime(Date time) {
        expirationTime = time;
    }

    public void setFQAN(String FQAN) {
        this.userFQAN = FQAN;
    }

    public void setInfo(String info) {
        this.info = info;
    }

    public void setLastUpdateTime(Date lastUpdateTime) {
        this.lastUpdateTime = lastUpdateTime;
    }

    public void setLocalUser(String localUser) {
        this.localUser = localUser;
    }

    public void setLocalUserGroup(String localUserGroup) {
        this.localUserGroup = localUserGroup;
    }

    public void setProxy(String proxy) {
        this.proxy = proxy;
    }

    public void setStartTime(Date startTime) {
        this.startTime = startTime;
    }

    public void setVO(String vO) {
        userVO = vO;
    }

    public void setVOMSAttributes(List<String> vomsAttributes) {
        this.vomsAttributes = vomsAttributes;
    }

    public String toString() {
        return info;
    }
}
